@@ -29,6 +29,7 @@ import pango
 from series import SeriesManager, Show, Episode
 from lib import constants
 from settings import Settings
+from asyncworker import AsyncWorker, AsyncItem
 
 _ = gettext.gettext
 
@@ -55,18 +56,29 @@ class MainWindow(hildon.StackableWindow):
         _ = language.gettext
         
         self.series_manager = SeriesManager()
-        self.series_manager.load(constants.SF_DB_FILE)
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        load_shows_item = AsyncItem(self.series_manager.load,
+                                    (constants.SF_DB_FILE,))
         self.series_manager.connect('show-list-changed',
                                     self._show_list_changed_cb)
         self.series_manager.connect('get-full-show-complete',
                                     self._get_show_complete_cb)
+        self.series_manager.connect('update-show-episodes-complete',
+                                    self._update_show_complete_cb)
+        self.series_manager.connect('update-shows-call-complete',
+                                    self._update_all_shows_complete_cb)
         
         self.settings = Settings()
-        self.settings.load(constants.SF_CONF_FILE)
+        load_conf_item = AsyncItem(self.settings.load,
+                                    (constants.SF_CONF_FILE,),
+                                    self._load_finished)
+        self.request = AsyncWorker()
+        self.request.queue.put(load_shows_item)
+        self.request.queue.put(load_conf_item)
+        self.request.start()
 
         self.shows_view = ShowsSelectView()
         self.shows_view.connect('row-activated', self._row_activated_cb)
-        self.shows_view.set_shows(self.series_manager.series_list)
         self.set_title(constants.SF_NAME)
         self.set_app_menu(self._create_menu())
         area = hildon.PannableArea()
@@ -75,7 +87,13 @@ class MainWindow(hildon.StackableWindow):
         
         self.connect('delete-event', self._exit_cb)
         self._update_delete_menu_visibility()
-    
+
+    def _load_finished(self, dummy_arg, error):
+        self.shows_view.set_shows(self.series_manager.series_list)
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
+        self.request = None
+        self._update_delete_menu_visibility()
+
     def _create_menu(self):
         menu = hildon.AppMenu()
         
@@ -88,7 +106,12 @@ class MainWindow(hildon.StackableWindow):
         self.delete_menu.set_label(_('Delete Shows'))
         self.delete_menu.connect('clicked', self._delete_shows_cb)
         menu.append(self.delete_menu)
-        
+
+        self.update_all_menu = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.update_all_menu.set_label(_('Update All'))
+        self.update_all_menu.connect('clicked', self._update_all_shows_cb)
+        menu.append(self.update_all_menu)
+
         menu.show_all()
         return menu
     
@@ -154,8 +177,21 @@ class MainWindow(hildon.StackableWindow):
         new_show_dialog.destroy()
     
     def _exit_cb(self, window, event):
-        self.series_manager.save(constants.SF_DB_FILE)
-        self.settings.save(constants.SF_CONF_FILE)
+        if self.request:
+            self.request.stop()
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        save_shows_item = AsyncItem(self.series_manager.save,
+                               (constants.SF_DB_FILE,))
+        save_conf_item = AsyncItem(self.settings.save,
+                               (constants.SF_CONF_FILE,),
+                               self._save_finished_cb)
+        async_worker = AsyncWorker()
+        async_worker.queue.put(save_shows_item)
+        async_worker.queue.put(save_conf_item)
+        async_worker.start()
+
+    def _save_finished_cb(self, dummy_arg, error):
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
         gtk.main_quit()
 
     def _show_list_changed_cb(self, series_manager):
@@ -164,10 +200,34 @@ class MainWindow(hildon.StackableWindow):
         return False
     
     def _update_delete_menu_visibility(self):
-        if len(self.series_manager.series_list):
-            self.delete_menu.show()
-        else:
+        if not self.series_manager.series_list or self.request:
             self.delete_menu.hide()
+            self.update_all_menu.hide()
+        else:
+            self.delete_menu.show()
+            self.update_all_menu.show()
+
+    def _update_all_shows_cb(self, button):
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        self.request = self.series_manager.update_all_shows_episodes()
+        self.set_sensitive(False)
+        self._update_delete_menu_visibility()
+
+    def _update_all_shows_complete_cb(self, series_manager, show, error):
+        self._show_list_changed_cb(self.series_manager)
+        if self.request:
+            if error:
+                show_information(self, _('Please verify your internet connection '
+                                         'is available'))
+            else:
+                show_information(self, _('Finished updating the shows'))
+        self.request = None
+        self.set_sensitive(True)
+        self._update_delete_menu_visibility()
+        hildon.hildon_gtk_window_set_progress_indicator(self, False)
+
+    def _update_show_complete_cb(self, series_manager, show, error):
+        show_information(self, _('Updated "%s"') % show.name)
 
 class DeleteView(hildon.StackableWindow):
     
@@ -253,19 +313,34 @@ class ShowsSelectView(gtk.TreeView):
             return
         seasons = len(show.get_seasons())
         if seasons:
-            show_info = '<small><span foreground="%s">' % constants.SECONDARY_TEXT_COLOR
+            color = get_color(constants.SECONDARY_TEXT_COLOR)
+            episodes_info = show.get_episodes_info()
+            episodes_to_watch = episodes_info['episodes_to_watch']
+            next_episode = episodes_info['next_episode']
+            if next_episode and next_episode.already_aired():
+                color = get_color(constants.ACTIVE_TEXT_COLOR) 
+            show_info = '<small><span foreground="%s">' % color
             show_info += gettext.ngettext('%s season', '%s seasons', seasons) \
                          % seasons
             if show.is_completely_watched():
                 show_info += ' | ' + _('Completely watched')
             else:
-                episodes_to_watch = len([episode for episode in show.episode_list \
-                                    if not episode.watched])
                 if episodes_to_watch:
+                    n_episodes_to_watch = len(episodes_to_watch)
                     show_info += ' | ' + gettext.ngettext('%s episode not watched',
                                                           '%s episodes not watched',
-                                                          episodes_to_watch) \
-                                                          % episodes_to_watch
+                                                          n_episodes_to_watch) \
+                                                          % n_episodes_to_watch
+                    if next_episode:
+                        next_air_date = next_episode.air_date
+                        if next_air_date:
+                            show_info += ' | ' + _('<i>Next air date:</i> %s') % \
+                                         next_episode.get_air_date_text()
+                        else:
+                            show_info += ' | ' + _('<i>Next to watch:</i> %s') % \
+                                         next_episode
+                        if next_episode.already_aired():
+                            color = get_color(constants.ACTIVE_TEXT_COLOR)
                 else:
                     show_info += ' | ' + _('No episodes to watch')
             show_info += '</span></small>'
@@ -295,21 +370,32 @@ class SeasonsView(hildon.StackableWindow):
         seasons = self.show.get_seasons()
         self.seasons_select_view.set_seasons(seasons)
         self.seasons_select_view.connect('row-activated', self._row_activated_cb)
+        self.connect('delete-event', self._delete_event_cb)
         
         seasons_area = hildon.PannableArea()
         seasons_area.add(self.seasons_select_view)
         self.add(seasons_area)
-    
+
+        self.request = None
+        self._update_menu_visibility()
+
+    def _delete_event_cb(self, window, event):
+        if self.request:
+            self.request.stop()
+            self.request = None
+        return False
+
     def _row_activated_cb(self, view, path, column):
         season = self.seasons_select_view.get_season_from_path(path)
         episodes_view = EpisodesView(self.settings, self.show, season)
         episodes_view.connect('delete-event', self._update_series_list_cb)
         episodes_view.connect('episode-list-changed', self._update_series_list_cb)
         episodes_view.show_all()
-    
+
     def _update_series_list_cb(self, widget, event = None):
         seasons = self.show.get_seasons();
         self.seasons_select_view.set_seasons(seasons)
+        self._update_menu_visibility()
     
     def _create_menu(self):
         menu = hildon.AppMenu()
@@ -325,10 +411,10 @@ class SeasonsView(hildon.StackableWindow):
         menu.append(button)
         
         if str(self.show.thetvdb_id) != '-1':
-            button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-            button.set_label(_('Update Show'))
-            button.connect('clicked', self._update_series_cb)
-            menu.append(button)
+            self.update_menu = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+            self.update_menu.set_label(_('Update Show'))
+            self.update_menu.connect('clicked', self._update_series_cb)
+            menu.append(self.update_menu)
         
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('New Episode'))
@@ -337,12 +423,19 @@ class SeasonsView(hildon.StackableWindow):
         
         menu.show_all()
         return menu
-    
+
+    def _update_menu_visibility(self):
+        if self.request or not self.show.get_seasons():
+            self.update_menu.hide()
+        else:
+            self.update_menu.show()
+
     def _update_series_cb(self, button):
-        self.series_manager.update_show_episodes(self.show)
+        self.request = self.series_manager.update_show_episodes(self.show)
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
         self.set_sensitive(False)
         show_information(self, _('Updating show. Please wait...'))
+        self._update_menu_visibility()
     
     def _show_info_cb(self, button):
         dialog = gtk.Dialog(parent = self)
@@ -392,6 +485,7 @@ class SeasonsView(hildon.StackableWindow):
             episode.director = episode_info['director']
             episode.writer = episode_info['writer']
             episode.rating = episode_info['rating']
+            episode.air_date = episode_info['air_date']
             episode.guest_stars = episode_info['guest_stars']
             self.show.update_episode_list([episode])
             seasons = self.show.get_seasons()
@@ -399,12 +493,12 @@ class SeasonsView(hildon.StackableWindow):
         new_episode_dialog.destroy()
     
     def _update_show_episodes_complete_cb(self, series_manager, show, error):
-        if error:
+        if error and self.request:
             error_message = ''
             if 'socket' in str(error).lower():
                 error_message = '\n ' + _('Please verify your internet connection '
                                           'is available')
-            show_information((self, error_message))
+            show_information(self, error_message)
         elif show == self.show:
             seasons = self.show.get_seasons()
             model = self.seasons_select_view.get_model()
@@ -413,6 +507,8 @@ class SeasonsView(hildon.StackableWindow):
                 self.seasons_select_view.set_seasons(seasons)
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
         self.set_sensitive(True)
+        self.request = None
+        self._update_menu_visibility()
     
 class SeasonSelectView(gtk.TreeView):
     
@@ -455,14 +551,16 @@ class SeasonSelectView(gtk.TreeView):
             name = _('Special')
         else:
             name = _('Season %s') % season
-        episodes = self.show.get_episode_list_by_season(season)
-        episodes_to_watch = [episode for episode in episodes \
-                            if not episode.watched]
+        info = self.show.get_episodes_info(season)
+        episodes = info['episodes']
+        episodes_to_watch = info['episodes_to_watch']
+        next_episode = info['next_episode']
         season_info = ''
+        color = get_color(constants.SECONDARY_TEXT_COLOR)
         if not episodes_to_watch:
             if episodes:
                 name = '<span foreground="%s">%s</span>' % \
-                        (constants.SECONDARY_TEXT_COLOR, name)
+                        (get_color(constants.SECONDARY_TEXT_COLOR), name)
                 season_info = _('Completely watched')
         else:
             number_episodes_to_watch = len(episodes_to_watch)
@@ -470,19 +568,18 @@ class SeasonSelectView(gtk.TreeView):
                                            '%s episodes not watched',
                                            number_episodes_to_watch) \
                                            % number_episodes_to_watch
-            sorted_episodes_to_watch = [episode.episode_number for episode in \
-                                        episodes_to_watch]
-            sorted_episodes_to_watch.sort()
-            next_episode = None
-            for episode in episodes_to_watch:
-                if episode.episode_number == sorted_episodes_to_watch[0]:
-                    next_episode = episode
-                    break
-            season_info += ' | ' + _('<i>Next to watch:</i> %s') % episode
+            if next_episode:
+                next_air_date = next_episode.air_date
+                if next_air_date:
+                    season_info += ' | ' + _('<i>Next air date:</i> %s') % next_episode.get_air_date_text()
+                else:
+                    season_info += ' | ' + _('<i>Next to watch:</i> %s') % next_episode
+                if next_episode.already_aired():
+                    color = get_color(constants.ACTIVE_TEXT_COLOR)
         renderer.set_property('markup',
                               '<b>%s</b>\n'
                               '<span foreground="%s">%s</span>' % \
-                              (name, constants.SECONDARY_TEXT_COLOR, season_info))
+                              (name, color, season_info))
         renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
 
 class NewShowDialog(gtk.Dialog):
@@ -596,6 +693,7 @@ class NewEpisodeDialog(gtk.Dialog):
         
         self.episode_director = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.episode_writer = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.episode_air_date = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.episode_rating = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.episode_guest_stars = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
         
@@ -613,6 +711,7 @@ class NewEpisodeDialog(gtk.Dialog):
         
         fields = [(_('Director:'), self.episode_director),
                   (_('Writer:'), self.episode_writer),
+                  (_('Original air date:'), self.episode_air_date),
                   (_('Rating:'), self.episode_rating),
                   (_('Guest Stars:'), self.episode_guest_stars),
                  ]
@@ -643,6 +742,7 @@ class NewEpisodeDialog(gtk.Dialog):
                 'number': self.episode_number.get_selector().get_entry().get_text(),
                 'director': self.episode_director.get_text(),
                 'writer': self.episode_writer.get_text(),
+                'air_date': self.episode_air_date.get_text(),
                 'rating': self.episode_rating.get_text(),
                 'guest_stars': self.episode_guest_stars.get_text()}
         return info
@@ -658,6 +758,7 @@ class EditEpisodeDialog(NewEpisodeDialog):
         self.episode_number.get_selector().get_entry().set_text(str(episode.episode_number))
         self.episode_director.set_text(episode.director)
         self.episode_writer.set_text(str(episode.writer))
+        self.episode_air_date.set_text(str(episode.air_date))
         self.episode_rating.set_text(episode.rating)
         self.episode_guest_stars.set_text(str(episode.guest_stars))
 
@@ -775,6 +876,10 @@ class EpisodesView(hildon.StackableWindow):
 
 class EpisodesCheckView(gtk.TreeView):
     
+    EPISODE_CHECK_COLUMN = 0
+    EPISODE_NAME_COLUMN = 1
+    EPISODE_OBJECT_COLUMN = 2
+    
     def __init__(self):
         super(EpisodesCheckView, self).__init__()
         model = gtk.ListStore(bool, str, gobject.TYPE_PYOBJECT)
@@ -783,14 +888,17 @@ class EpisodesCheckView(gtk.TreeView):
         column = gtk.TreeViewColumn('Watched', self.watched_renderer)
         column.add_attribute(self.watched_renderer, "active", 0)
         self.append_column(column)
-        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text = 1)
+        episode_renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn('Name', episode_renderer, text = 1)
+        column.set_cell_data_func(episode_renderer,
+                                  self._episode_select_view_data_func)
         self.append_column(column)
         self.set_model(model)
         self.get_model().set_sort_func(2, self._sort_func)
     
     def _sort_func(self, model, iter1, iter2):
-        episode1 = model.get_value(iter1, 2)
-        episode2 = model.get_value(iter2, 2)
+        episode1 = model.get_value(iter1, self.EPISODE_OBJECT_COLUMN)
+        episode2 = model.get_value(iter2, self.EPISODE_OBJECT_COLUMN)
         if episode1 == None or episode2 == None:
             return 0
         if episode1.episode_number < episode2.episode_number:
@@ -807,14 +915,16 @@ class EpisodesCheckView(gtk.TreeView):
     def get_episode_from_path(self, path):
         model = self.get_model()
         iter = model.get_iter(path)
-        episode = model.get_value(iter, 2)
+        episode = model.get_value(iter, self.EPISODE_OBJECT_COLUMN)
         return episode
     
     def sort_descending(self):
-        self.get_model().set_sort_column_id(2, gtk.SORT_DESCENDING)
+        self.get_model().set_sort_column_id(self.EPISODE_OBJECT_COLUMN,
+                                            gtk.SORT_DESCENDING)
     
     def sort_ascending(self):
-        self.get_model().set_sort_column_id(2, gtk.SORT_ASCENDING)
+        self.get_model().set_sort_column_id(self.EPISODE_OBJECT_COLUMN,
+                                            gtk.SORT_ASCENDING)
     
     def select_all(self):
         for path in self.get_model():
@@ -822,7 +932,19 @@ class EpisodesCheckView(gtk.TreeView):
     
     def select_none(self):
         for path in self.get_model() or []:
-            path[0] = path[2].watched = False
+            path[self.EPISODE_CHECK_COLUMN] = \
+                path[self.EPISODE_OBJECT_COLUMN].watched = False
+    
+    def _episode_select_view_data_func(self, column, renderer, model, iter):
+        episode = model.get_value(iter, self.EPISODE_OBJECT_COLUMN)
+        color = get_color(constants.SECONDARY_TEXT_COLOR)
+        if not episode.watched and episode.already_aired():
+            color = get_color(constants.ACTIVE_TEXT_COLOR)
+        renderer.set_property('markup',
+                              '<span foreground="%s">%s\n'
+                              '%s</span>' % \
+                              (color, episode, episode.get_air_date_text()))
+        renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
 
 class EpisodeView(hildon.StackableWindow):
     
@@ -849,6 +971,8 @@ class EpisodeView(hildon.StackableWindow):
         self.infotextview.set_title(self.episode.name)
         self.infotextview.add_field(self.episode.overview)
         self.infotextview.add_field('\n')
+        self.infotextview.add_field(self.episode.get_air_date_text(),
+                                    _('Original air date'))
         self.infotextview.add_field(self.episode.director, _('Director'))
         self.infotextview.add_field(self.episode.writer, _('Writer'))
         self.infotextview.add_field(self.episode.guest_stars, _('Guest Stars'))
@@ -876,6 +1000,7 @@ class EpisodeView(hildon.StackableWindow):
             self.episode.overview = episode_info['overview']
             self.episode.season_number = episode_info['season']
             self.episode.episode_number = episode_info['number']
+            self.episode.air_date = episode_info['air_date']
             self.episode.director = episode_info['director']
             self.episode.writer = episode_info['writer']
             self.episode.rating = episode_info['rating']
@@ -1084,3 +1209,15 @@ def show_information(parent, message):
     hildon.hildon_banner_show_information(parent,
                                           '',
                                           message)
+
+def get_color(color_name):
+    # Adapted from gPodder
+    settings = gtk.settings_get_default()
+    if not settings:
+        return None
+    color_style = gtk.rc_get_style_by_paths(settings,
+                                            'GtkButton',
+                                            'osso-logical-colors',
+                                            gtk.Button)
+    return color_style.lookup_color(color_name).to_string()
+
